<?php

/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Malin Lorch <mlorch@liquidweb.com>
 * @date    Aug 28, 2020
 */

require_once(__DIR__ . '/../Acronis.php');
/**
 *
 * @package Interworx
 * @subpackage Plugin_Acronis
 */
class Exception_Acronis_ApiFailure extends Exception_Acronis
{
    private $_responseCode;
    private $_responseMessage;
    private $_message;

    private $_errorCode;

    private $_errorCodes = array(
        400 => AcronisReply::ERROR_INTERNAL,
        401 => AcronisReply::ERROR_API_INVALID_CREDENTIALS,
        403 => AcronisReply::ERROR_API_INVALID_USER_TYPE,
        404 => AcronisReply::ERROR_API_INVALID_ENDPOINT,
        500 => AcronisReply::ERROR_API_NOT_RESPONSIVE,
        504 => AcronisReply::ERROR_API_BAD_GATEWAY,
        1000 => AcronisReply::ERROR_API_INVALID_CREDENTIALS,
        1002 => AcronisReply::ERROR_API_INVALID_USER_TYPE
    );

    /**
     * @param array  $response
     * @param string $apiEndpoint
     */
    public function __construct(array $response, $apiEndpoint)
    {
        $error = $response['error'];

        $this->_responseCode = $error['code'];
        $this->_responseMessage = $error['message'];
        $this->_message = "ERROR Acronis Api. Api endpoint: $apiEndpoint. Response code: $this->_responseCode. Error message: $this->_responseMessage";

        $cookieRequestFailed = strpos($this->_responseMessage, 'Cookie request failed') !== false;
        if ($cookieRequestFailed) {
            $this->_errorCode = AcronisReply::ERROR_API_INVALID_COOKIE;
        }

        parent::__construct($this->_message);
    }

    /**
     * Get error code.
     *
     * @return integer
     */
    public function getErrorCode()
    {
        if (is_null($this->_errorCode)) {
            $this->_errorCode = $this->_getErrorCodeByResponseCode();
        }

        return $this->_errorCode;
    }

    /**
     * Get error code based on acronis response code.
     */
    private function _getErrorCodeByResponseCode()
    {
        $errorCode = $this->_errorCodes[$this->_responseCode];

        if (is_null($errorCode)) {
            return AcronisReply::ERROR_API;
        }

        return $errorCode;
    }
}
