<?php

/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2022 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @author     Malin Lorch <mlorch@liquidweb.com>
 * @date       Jun 29, 2022
 */

class Model_Api_Private_Ams_Archives extends Model_Api
{
    const API_ENDPOINT = '/bc/api/ams/archives';

    /**
     * Get directory content from acronis api.
     *
     * @param string $legacyId
     * @param string $directoryId
     * @return array
     */
    public function getDirectoryContent($legacyId, $directoryId = null)
    {
        $postData = $this->_getDirectoryContentPostData($legacyId, $directoryId);
        $apiEndpoint = $this->_getApiEndpoint($postData);
        $config = array('data' => $postData);

        return $this->_getCookieClient()->sendAuthenticatedPostRequest($apiEndpoint, $config);
    }


    /**
     * Get post data.
     *
     * @param string $legacyId
     * @param string $directoryId
     * @return array
     */
    private function _getDirectoryContentPostData($legacyId, $directoryId = null)
    {
        $postData = array(
            'backupId' => $legacyId,
            'machineId' => $this->_getAgentId(),
            'maxRequestTimeout' => 600000,
            'timeout' => 30,
            'top' => 0,
            'type' => 'files'
        );

        if (isset($directoryId)) {
            $postData['id'] = $directoryId;
        }

        return $postData;
    }

    /**
     * Get recovery point url.
     *
     * @param array $postData
     */
    private function _getApiEndpoint(array $postData)
    {
        $query = http_build_query($postData);
        return self::API_ENDPOINT . "/%3CpatharchiveId%3E/backups/%3CpathbackupId%3E/items?$query";
    }
}
