<?php

/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2022 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @author     Malin Lorch <mlorch@liquidweb.com>
 * @date       Jul 08, 2022
 */

class Model_Api_Public_TaskManager_Tasks extends Model_Api
{
    public const TASK_TYPE_RECOVERY = 'recovery';
    public const TASK_TYPE_BACKUP = 'backup';

    public const TASK_VALID_TYPES = [
        self::TASK_TYPE_BACKUP,
        self::TASK_TYPE_RECOVERY
    ];

    const API_ENDPOINT = '/api/task_manager/v2/tasks';

    /**
     * @param string $policy_id
     * @return Model_ApiResponse_Task
     */
    public function getRecoveryByPolicyId($policy_id): Model_ApiResponse_Task
    {
        $query = http_build_query(array(
            'policyId' => $policy_id,
            'resourceId' => $this->_getInstanceId()
        ));

        $apiEndpoint = self::API_ENDPOINT . "?$query";
        $res = $this->_getOAuthClient()->sendGetRequest($apiEndpoint);
        $items = $res['items'];

        if (empty($items)) {
            throw new Exception_Acronis_InvalidInput($policy_id);
        }

        $task = $res['items'][0];
        return new Model_ApiResponse_Task($task);
    }

    /**
     * Get recoveries
     */
    public function getRecoveries(): array
    {
        return $this->_getTasksByType(self::TASK_TYPE_RECOVERY);
    }

    /**
     * Get backups
     *
     * Currently not in use but possibly something we can utilize?
     * Need to compare data with current endpoint for getting backups
     */
    public function getBackups()
    {
        return $this->_getTasksByType(self::TASK_TYPE_BACKUP);
    }

    /**
     * @param string $type
     */
    private function _getTasksByType($type)
    {
        if (!in_array($type, self::TASK_VALID_TYPES)) {
            throw new IWorx_Exception_DoingItWrong('Acronis: Task type is invalid: ' . $type);
        }

        $query = http_build_query(array(
            'resourceId' => $this->_getInstanceId(),
            'policyType' => $type,
            'order' => 'desc(startedAt)' // for backward compatibility
        ));

        $apiEndpoint = self::API_ENDPOINT . "?$query";
        $res = $this->_getOAuthClient()->sendGetRequest($apiEndpoint);
        return array_map(function ($task) {
            return new Model_ApiResponse_Task($task);
        }, $res['items']);
    }
}
