<?php

/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Malin Lorch <mlorch@liquidweb.com>
 * @date    Apr 23, 2020
 */

/**
 *
 *
 * @package Interworx
 * @subpackage Plugin_Acronis
 */
class Model_Cache
{
    const CACHE_DB_RESTORES = 'dbrestores.acronis';
    const CACHE_FILE_RESTORES = 'filerestores.acronis';
    const CACHE_INSTANCE_ID = 'instanceid.acronis';
    const CACHE_AGENT_ID = 'agentid.acronis';
    const CACHE_RESOURCE_ID = 'resourceid.acronis';
    const CACHE_PLAN_ID = 'planid.acronis';
    const CACHE_DEVICEINFO = 'deviceinfo.acronis';
    const CACHE_PLAN = 'plan.acronis';
    const CACHE_BACKUPS = 'backups.acronis';
    const CACHE_BACKUP = 'backup.acronis';
    const CACHE_DIRECTORIES = 'directories.acronis';
    const CACHE_AGENT_TENANT_ID = 'agenttenantid.acronis';
    const CACHE_DB_DUMPS = 'dbdumps.acronis';
    const CACHE_DIR_BY_PATH = 'backups.path.acronis';
    const CACHE_MACHINE_ID = 'machineid.acronis';
    const CACHE_LEGACY_IDS = 'legacyids.acronis';
    const CACHE_TENANT_AGENT_ID = 'tenantagentid.acronis';
    const CACHE_CREATED_AT = 'backupscreatedat.acronis';
    const CACHE_PRESCRIPT = 'prescript.acronis';
    const CACHE_EXPIRATION = 'expire.acronis';

    const TWENTY_DAYS = '1728000';

    /**
     * check if exists in cache.
     *
     * @param string $key
     */
    public function isCached($key)
    {
        return $this->_getCache($key) !== null;
    }

    /**
     * @param string $agentId
     */
    public function cacheAgentId($agentId)
    {
        $this->_setCache(self::CACHE_AGENT_ID, $agentId);
    }

    /**
     * Get agent id.
     *
     * @return string
     */
    public function getAgentId()
    {
        return $this->_getCache(self::CACHE_AGENT_ID);
    }

    /**
     * Cache instance id.
     *
     * @param string $instanceId
     */
    public function cacheInstanceId($instanceId)
    {
        $this->_setCache(self::CACHE_INSTANCE_ID, $instanceId);
    }

    /**
     * Get instance id.
     *
     * @return string
     */
    public function getInstanceId()
    {
        return $this->_getCache(self::CACHE_INSTANCE_ID);
    }

    /**
     * Check if instance id exists in cache.
     *
     * @return boolean
     */
    public function instanceIdExists(): bool
    {
        return !is_null($this->getInstanceId());
    }

    /**
     * Cache resource id.
     *
     * @param string $resourceId
     */
    public function cacheResourceId($resourceId)
    {
        $this->_setCache(self::CACHE_RESOURCE_ID, $resourceId);
    }


    /**
     * Get resource id.
     *
     * @return string
     */
    public function getResourceId()
    {
        return $this->_getCache(self::CACHE_RESOURCE_ID);
    }

    /**
     * Cache device info.
     *
     * @param array $deviceInfo
     */
    public function cacheDeviceInfo(array $deviceInfo)
    {
        $this->_setTempCache(self::CACHE_DEVICEINFO, $deviceInfo);
    }

    /**
     * Get device info.
     *
     * @return array
     */
    public function getDeviceInfo()
    {
        return $this->_getCachedArray(self::CACHE_DEVICEINFO);
    }

    /**
     * Cache plan.
     *
     * @param array $plan
     */
    public function cachePlan(array $plan)
    {
        $this->_setTempCache(self::CACHE_PLAN, $plan);
    }

    /**
     * Get plan.
     *
     * @return array
     */
    public function getPlan()
    {
        return $this->_getCachedArray(self::CACHE_PLAN);
    }

    /**
     * Cache plan id.
     *
     * @param string $id
     */
    public function cachePlanId($id)
    {
        IW::getLog()->debug('Acronis: Caching new plan id: ' . $id);
        $this->_setCache(self::CACHE_PLAN_ID, $id);
    }

    /**
     * get plan id.
     *
     * @return string
     */
    public function getPlanId()
    {
        return $this->_getCache(self::CACHE_PLAN_ID);
    }

    /**
     * delete plan id.
     */
    public function deletePlanId()
    {
        IW::getLog()->debug('Acronis: Deleting cached plan id');
        $this->_deleteCache(self::CACHE_PLAN_ID);
    }

    /**
     * Get backups.
     *
     * @return array
     */
    public function getBackups()
    {
        return $this->_getCachedArray(self::CACHE_BACKUPS);
    }

    /**
     * Cache backups.
     *
     * @param array $backups
     */
    public function cacheBackups(array $backups)
    {
        $this->_setCache(self::CACHE_BACKUPS, $backups);
    }

    /**
     * Get agent tenant id.
     *
     * @return string
     */
    public function getAgentTenantId()
    {
        return $this->_getCache(self::CACHE_AGENT_TENANT_ID);
    }

    /**
     * cache agent tenant id.
     *
     * @param string $id
     */
    public function cacheAgentTenantId($id)
    {
        $this->_setCache(self::CACHE_AGENT_TENANT_ID, $id);
    }



    /**
     * Get backup by id.
     *
     * @param string $id
     * @return array
     */
    public function getBackupById($id)
    {
        $backups = $this->getBackups();
        if (empty($backups)) {
            throw new Exception('Could not get backup by id, no backups have been cached!');
        }

        foreach ($backups as $backup) {
            if ($backup['backupId'] === $id) {
                return $backup;
            }
        }
    }

    /**
     *
     * @param string $backupId
     */
    public function getDumpsInfo($backupId)
    {
        $dumps = $this->_getDumps();
        return $dumps[$backupId];
    }

    /**
     * Cache dumps.
     *
     * @param string $backupId
     * @param array  $dumpsToCache
     */
    public function cacheDumps($backupId, array $dumpsToCache)
    {
        $dumps = $this->_getDumps();
        if (!isset($dumps)) {
            $dumps = array();
        }

        $dumps[$backupId] = $dumpsToCache;
        $this->_setDumps($dumps);
    }

    /**
     * @return array
     */
    private function _getDumps()
    {
        return $this->_getCachedArray(self::CACHE_DB_DUMPS);
    }

    /**
     * Set db dumps.
     *
     * @param array $dumps
     */
    private function _setDumps(array $dumps)
    {
        $cacheForTwentyDays = '1728000';
        $this->_setCache(self::CACHE_DB_DUMPS, $dumps, $cacheForTwentyDays);
    }

    /**
     * Get directories.
     *
     * @return array
     */
    public function getDirectories()
    {
        return $this->_getCachedArray(self::CACHE_DIRECTORIES);
    }

    /**
     * Cache dir.
     *
     * @param string $backupId
     * @param array  $dir
     */
    public function cacheDir($backupId, array $dir)
    {
        $backupDirs = $this->_getBackupDirs($backupId);
        $backupDirs[] = $dir;
        $this->_replaceBackupDirs($backupId, $backupDirs);
    }

    /**
     * @param string $backupId
     * @param string $devicePath
     */
    public function getDirByDevicePath($backupId, $devicePath)
    {
        $backupDirs = $this->_getBackupDirs($backupId);


        if (!empty($backupDirs)) {
            $dir = $this->_getDirByFullPath($devicePath, $backupDirs);
            return $dir;
        }
    }


    /**
     * Get dir by path (acronis path/device path).
     *
     * @param string $path
     * @param array  $backupDirs
     * @return array
     */
    private function _getDirByFullPath($path, array $backupDirs)
    {
        foreach ($backupDirs as $dir) {
            if ($dir['fullPath'] === $path) {
                return $dir;
            }
        }
    }

    /**
     * Get directories.
     *
     * @return array
     */
    private function _getDirectories()
    {
        return $this->_getCachedArray(self::CACHE_DIRECTORIES);
    }

    /**
     * Get backup directories.
     *
     * @param string $backupId
     * @return array
     */
    private function _getBackupDirs($backupId)
    {
        $dirs = $this->_getDirectories();

        if (empty($dirs) || empty($dirs[$backupId])) {
            return array();
        }

        return $dirs[$backupId];
    }

    /**
     * Add backup to directory cache.
     *
     * @param string $backupId
     * @param array  $backupDirs
     */
    private function _replaceBackupDirs($backupId, array $backupDirs)
    {
        $dirs = $this->_getDirectories();
        if (empty($dirs)) {
            $dirs = array();
        }

        $dirs[$backupId] = $backupDirs;
        $this->_cacheDirectories($dirs);
    }

    /**
     * Cache directories.
     *
     * @param array $dirs
     */
    private function _cacheDirectories(array $dirs)
    {
        $this->_setCache(self::CACHE_DIRECTORIES, $dirs, self::TWENTY_DAYS);
    }

    /**
     * @param string $backupId
     * @param string $path
     */
    public function getDirContentByPath($backupId, $path)
    {
        $backup = $this->_getPathBackupByBackupId($backupId);
        return $backup[$path];
    }

    /**
     * Cached path includes device name.
     *
     * @param string $backupId
     * @param string $path
     * @param array  $content
     */
    public function cacheDirContentByPath($backupId, $path, array $content)
    {
        $backup = $this->_getPathBackupByBackupId($backupId);
        $backup[$path] = $content;

        $this->_savePathBackupByBackupId($backupId, $backup);
    }

    /**
     * @param string $id
     */
    private function _getPathBackupByBackupId($id)
    {
        $backups = $this->_getCachedArray(self::CACHE_DIR_BY_PATH);
        if (!isset($backups)) {
            $backups = array();
        }

        return $backups[$id] === null ? array() : $backups[$id];
    }

    /**
     * @param string $backupId
     * @param array  $backup
     */
    private function _savePathBackupByBackupId($backupId, array $backup)
    {
        $backups = $this->_getCachedArray(self::CACHE_DIR_BY_PATH);
        $backups = isset($backups) ? $backups : array();
        $backups[$backupId] = $backup;
        $this->_setCache(self::CACHE_DIR_BY_PATH, $backups);
    }

    /**
     * Get cached values.
     *
     * @param string $name
     */
    protected function _getCache($name)
    {
        return IW::Cache()->get($name);
    }


    /**
     * Format cached return object to assoc array.
     *
     * @param string $cacheName
     * @return array
     */
    protected function _getCachedArray($cacheName)
    {
        $cachedObject = $this->_getCache($cacheName);


        $assoc = true;
        return json_decode(json_encode($cachedObject), $assoc);
    }

    /**
     * Set permanent cache.
     *
     * @param string  $name
     * @param mixed   $data
     * @param integer $time
     */
    protected function _setCache($name, $data, $time = 0)
    {
        IW::Cache()->set($name, $data, $time);
    }

    /**
     * Set temporary cache. Will be cleared after 10 sec.
     *
     * @param string $name
     * @param mixed  $data
     */
    protected function _setTempCache($name, $data)
    {
        $seconds = 5;
        IW::Cache()->set($name, $data, $seconds);
    }


    /**
     * Delete cache.
     *
     * @param string $name
     */
    protected function _deleteCache($name)
    {
        IW::Cache()->delete($name);
    }

    /**
     * Delete all cached values.
     */
    public function deleteAll()
    {
        IW::getLog()->info('Acronis: deleting cache (excluding prescript data)');
        $all = $this->_getAllCacheConsts();
        foreach ($all as $cache) {
            if ($this->_isDeletable($cache)) {
                $this->_deleteCache($cache);
            }
        }
    }

    /**
     * do not delete prescript data as this can't be
     * collected from acronis api. Use action to clear
     * any unmapped data that does not correlate with a backup.
     *
     * prescript data might have been merged with backup - so backups
     * should not be deleted either.
     *
     * @param string $cache
     */
    private function _isDeletable($cache)
    {
        return $cache !== self::CACHE_BACKUP && $cache !== self::CACHE_PRESCRIPT;
    }

    /**
     * @param string $machineId
     */
    public function cacheMachineId($machineId)
    {
        if (isset($machineId)) {
            IW::Cache()->set(self::CACHE_MACHINE_ID, $machineId);
        }
    }

    /**
     * @param string $backupId
     * @param string $legacyId
     */
    public function cacheLegacyId($backupId, $legacyId)
    {
        if (!isset($backupId) || !isset($legacyId)) {
            throw new Exception('backup id or legacy id not set!');
        }

        $legacyIds = $this->_getLegacyIds();
        $legacyIds[$backupId] = $legacyId;

        $this->_setCache(self::CACHE_LEGACY_IDS, $legacyIds);
    }


    /**
     * Legacy id is different for each backup.
     *
     * @return object
     */
    private function _getLegacyIds()
    {
        $cache = $this->_getCache(self::CACHE_LEGACY_IDS);
        $legacyIds = isset($cache) ? (array)$cache : array();

        return $legacyIds;
    }

    /**
     * @param string $backupId
     * @return string
     */
    public function getLegacyId($backupId)
    {
        $legacyIds = $this->_getLegacyIds();
        if (is_null($legacyIds[$backupId])) {
            throw new Exception('Acronis Cache: Legacy id missing in cache');
        }
        return $legacyIds[$backupId];
    }

    /**
     * @param string $backupId
     * @param string $createdAt
     */
    public function cacheBackupCreatedAt($backupId, $createdAt)
    {
        if (!isset($backupId) || !isset($createdAt)) {
            throw new Exception('backup id or created at missing!');
        }

        $backupsCreatedAt = $this->_getBackupsCreatedAt();
        $backupsCreatedAt[$backupId] = $createdAt;

        $this->_setCache(self::CACHE_CREATED_AT, $backupsCreatedAt);
    }

    /**
     * get creation dates for backups.
     *
     * @return array
     */
    private function _getBackupsCreatedAt()
    {
        $cache = $this->_getCache(self::CACHE_CREATED_AT);
        $backupsCreatedAt = isset($cache) ? (array)$cache : array();

        return $backupsCreatedAt;
    }


    /**
     * @param string $backupId
     * @return string
     */
    public function getCreatedAt($backupId)
    {
        $backupsCreatedAt = $this->_getBackupsCreatedAt();
        return $backupsCreatedAt[$backupId];
    }

    /**
     * @param string $tenantAgentId
     */
    public function cacheTenantAgentId($tenantAgentId)
    {
        if (isset($tenantAgentId)) {
            IW::Cache()->set(self::CACHE_AGENT_TENANT_ID, $tenantAgentId);
        }
    }

    /**
     * @return string
     */
    public function getTenantAgentId()
    {
        $tenantAgentId = $this->_getCache(self::CACHE_AGENT_TENANT_ID);
        return $tenantAgentId;
    }


    /**
     * Get all const in class.
     *
     * @return array
     */
    private function _getAllCacheConsts()
    {
        $thisClass = new ReflectionClass(__CLASS__);
        return $thisClass->getConstants();
    }
}
