<?php

/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2021 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @author     Malin Lorch <mlorch@liquidweb.com>
 * @date       Jun 21, 2021
 */

/**
 * @package InterWorx
 * @subpackage
 */
class Model_Cache_Restore_Files extends Model_Cache_Restore
{
    const CACHE_KEY = Model_Cache::CACHE_FILE_RESTORES;

    /**
     * @return string
     */
    protected function _getCacheKey()
    {
        return self::CACHE_KEY;
    }

    /**
     * @return string
     */
    protected function _getCacheName()
    {
        return 'filerestores';
    }

    /**
     * @param string $planId
     * @param array  $files
     * @param string $unixuser
     */
    public static function getCacheFormat($planId, array $files, $unixuser)
    {
        return array(
            self::PLAN_ID => $planId,
            self::TIME => Model_Acronis::getDateString(time()),
            self::DIRECTORIES => $files,
            self::UNIXUSER => $unixuser
        );
    }

    /**
     * @param string $user
     * @return array
     */
    public function getByUser($user)
    {
        $all = $this->getAll();
        $userRestores = array();

        foreach ($all as $restore) {
            if ($restore[self::UNIXUSER] === $user) {
                $userRestores[] = $restore;
            }
        }

        return $userRestores;
    }
}
