<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Malin Lorch <mlorch@liquidweb.com>
 * @date    Dec 22, 2020
 */

/**
 *
 */
class Model_RecoveryPlan_Plan extends Model_Acronis
{
    private $_planId;
    private $_state;
    private $_progress;
    private $_uuid;
    private $_codeType;
    private $_codeError;
    private $_errorDetected;
    private $_errorMessage;
    private $_errorDetails;

    /**
     * @param Model_ApiResponse_Task $Task
     */
    public function __construct(Model_ApiResponse_Task $Task)
    {
        $this->_taskId = $Task->getId();
        $this->_uuid = $Task->getUuid();
        $this->_codeType = $Task->getCodeType();
        $this->_codeError = $Task->getCodeError();
        $this->_progress = $Task->getProgressPercent();
        $this->_state = $Task->getState();

        $this->_errorDetected = $Task->errorDetected();
        $this->_errorMessage = $Task->getErrorMessage();
        $this->_errorDetails = $Task->getErrorDetails();
    }

    /**
     * @return array
     */
    public function getErrorMessage()
    {
        if ($this->_errorDetected) {
            return array(
                'message' => $this->_errorMessage
            );
        }
    }

    /**
     * @return array
     */
    public function getError()
    {
        if ($this->_errorDetected) {
            return array(
                'message' => $this->_errorMessage,
                'context' => $this->_errorDetails
            );
        }
    }

    /**
     * Get progress.
     *
     * @return string
     */
    public function getProgress()
    {
        return $this->_progress;
    }

    /**
     * Get state.
     *
     * @return string
     */
    public function getState()
    {
        return $this->_state;
    }

    /**
     * get unique identifier (also refer to it as recovery id)
     *
     * @return string
     */
    public function getUuid()
    {
        return $this->_uuid;
    }

    /**
     * get plan id.
     *
     * @return string
     */
    public function getPlanId()
    {
        return $this->_planId;
    }

    /**
     * check if plan status is 'completed'
     *
     * @return boolean
     */
    public function recoveryComplete()
    {
        return $this->_state === 'completed';
    }

    /**
     * check if plan status is 'paused'
     *
     * @return boolean
     */
    public function recoveryPaused()
    {
        return $this->_state === 'paused';
    }

    /**
     * @return boolean
     */
    public function getCodeType()
    {
        return $this->_codeType;
    }

    /**
     * @return boolean
     */
    public function getCodeError()
    {
        return $this->_codeError;
    }
}
