<?php

/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2021 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @author     Malin Lorch <mlorch@liquidweb.com>
 * @date       Aug 30, 2021
 */
require_once(__DIR__ . '/../../Nodeworx.php');

/**
 * @package InterWorx
 * @subpackage
 */
class NwRunBackupTest extends Acronis_NodeworxTest
{
    /**
     * Remove parent dir.
     */
    public function tearDown()
    {
        if (Model_AcronisFileSys::isDir(self::PARENT_DIR_PATH_TEST)) {
            IWorxFileSys::rm(self::PARENT_DIR_PATH_TEST, true, true);
        }
    }


    /**
     * Test activate plan if not already activated. Is tested more thoroughly later on.
     */
    public function testActivatePlan()
    {
        if (!$this->_isActivated()) {
            echo __FUNCTION__;

            $Reply = $this->_sendCtrlRequest('activatePlan');
            $this->assertReplySuccessful($Reply);
            $this->_awaitActivation();
        }
    }


    /**
     * Test run backup. Create test directories before running backup.
     */
    public function testRunBackup()
    {
        echo __FUNCTION__;
        // await acronis api after activating new plan.
        sleep(60);

        $OldBackup = $this->_getBackup();

        $this->_createAccountAndDb();
        $this->_createTestDirectories(self::DIR_CONTENT_EXPECTED_COUNT);

        $Reply = $this->_sendCtrlRequest('runBackup');
        $this->assertReplySuccessful($Reply);


        if (is_null($OldBackup)) {
            $this->_awaitInitialBackup();
        } else {
            $this->_awaitBackup($OldBackup->getBackupId());
        }
    }

    /**
     * Create sw account and dbs.
     */
    private function _createAccountAndDb()
    {
        $SW = new SiteWorx();
        $SW->populateUniqname('acronistest');

        if (!$SW->exists()) {
            $this->_createAcct();
        }

        $this->_createSwDatabases();
        $this->_createSystemDatabase();
    }

    /**
     * create sw account & dbs.
     */
    private function _createAcct()
    {
        $Command = new Command('/home/interworx/bin/php /home/interworx/plugins/acronis/bin/acronisdbseeder.php --createAccountAndDatabases');
        $Command->runAsRoot();
        $Command->exec();

        if (!$Command->wasSuccessful()) {
            $this->fail('UNABLE TO CREATE ACCOUNT AND DB!');
        }
    }

    /**
     * Create sw dbs
     */
    private function _createSwDatabases()
    {
        $Command = new Command('/home/interworx/bin/php /home/interworx/plugins/acronis/bin/acronisdbseeder.php  --createSwDatabases');
        $Command->runAsRoot();
        $Command->exec();

        if (!$Command->wasSuccessful()) {
            $this->fail('UNABLE TO CREATE SW DBS!');
        }
    }

    /**
     * Create system db
     */
    private function _createSystemDatabase()
    {
        $Command = new Command('/home/interworx/bin/php /home/interworx/plugins/acronis/bin/acronisdbseeder.php --createSystemDatabase');
        $Command->runAsRoot();
        $Command->exec();

        if (!$Command->wasSuccessful()) {
            $this->fail('UNABLE TO CREATE SYSTEM DB!');
        }
    }


    /**
     * Create test directories, used before running backup.
     *
     * @param integer $amount
     */
    private function _createTestDirectories($amount)
    {
        for ($i = 0; $i < $amount; $i++) {
            $dirName = "testdir_$i";
            Model_AcronisFileSys::createDir(self::PARENT_DIR_PATH_TEST . '/' . $dirName);
        }
    }
}
