<?php

/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2022 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @author     Malin Lorch <mlorch@liquidweb.com>
 * @date       Jun 06, 2022
 */

/**
 * api output mocked from the plan that nexcess use for all of their production servers
 *
 * it's named "30 Days"
 */
class Policy30DaysTest extends IWorx_Framework_TestCase
{

    /**
     * Test root policy class - make sure api data is parsed correctly.
     */
    public function testTotalPolicy()
    {
        $policies_json = $this->_getMockedApiResponse('applied_policies_activated.json');
        $context_json = $this->_getMockedApiResponse('context.json');

        $policies = json_decode($policies_json, true);
        $context = json_decode($context_json, true);

        $Policy = new Model_Backups_Plan_Policy_Total($policies, $context);

        $expected_type = 'policy.protection.total';
        $expected_name = '30 Days';
        $expected_id = '3044c6a6-e729-4c65-b4e9-453c4bc0ea41';

        $this->assertSame($expected_type, $Policy->getType(), "expected type: $expected_type");
        $this->assertSame($expected_name, $Policy->getName(), "expected name: $expected_name");
        $this->assertSame($expected_id, $Policy->getId(), "expected id: $expected_id");

        $BackupPolicy = $Policy->getBackupMachinePolicy();
        $expected_class = Model_Backups_Plan_Policy_BackupMachine::class;
        $this->assertInstanceOf($expected_class, $BackupPolicy, "Expected backup policy to be instance of $expected_class");

        return $BackupPolicy;
    }


    /**
     * Get mocked api response from file
     *
     * @param string $file_name
     * @return string
     */
    protected function _getMockedApiResponse($file_name)
    {
        return file_get_contents(__DIR__ . "/../../resources/api_mock_response/policies/30days/$file_name");
    }

    /**
     * @depends testTotalPolicy
     * @param Model_Backups_Plan_Policy_BackupMachine $Policy
     */
    public function testBackupPolicy(Model_Backups_Plan_Policy_BackupMachine $Policy)
    {
        $expected_type = 'policy.backup.machine';
        $expected_name = '30 Days';
        $expected_id = '9157bb83-e2a6-45f9-a951-0e4007e08207';

        $this->assertSame($expected_type, $Policy->getType(), "expected type: $expected_type");
        $this->assertSame($expected_name, $Policy->getName(), "expected name: $expected_name");
        $this->assertSame($expected_id, $Policy->getId(), "expected id: $expected_id");

        $this->assertSame($this->_getExpectedPreDataCommands(), $Policy->getPreDataCommands(), "Not getting the correct pre DATA commands!");
        $this->assertSame($this->_getExpectedPostCommands(), $Policy->getPostCommands(), "Not getting the correct post commands!");
        $this->assertSame($this->_getExpectedPreCommands(), $Policy->getPreCommands(), "Not getting the correct pre commands!");

        $this->assertArrayHasKey('type', $Policy->getTime());
        $this->assertArrayHasKey('hour', $Policy->getTime());
        $this->assertArrayHasKey('minute', $Policy->getTime());
        $this->assertSame('daily', $Policy->getTime()['type']);

        $expected_weekdays = ['mon', 'tue', 'wed', 'thu', 'fri', 'sat', 'sun'];
        $this->assertSame($expected_weekdays, $Policy->getDays(), 'Expected days to be mon-sun in array');

        $this->assertSame(22, $Policy->getTime()['hour']);
        $this->assertSame(30, $Policy->getTime()['minute']);
    }

    /**
     * Validate expected pre data commands
     *
     * @return array
     */
    private function _getExpectedPreDataCommands(): array
    {
        return array(
            "type" => "pre_data_command",
            "enabled" => true,
            "cmd" => "/var/lib/Acronis/AgentCommData/nx-pre-capture-data.sh",
            "args" => "",
            "workdir" => "/var/lib/Acronis/AgentCommData/",
            "wait" => true,
            "stop_on_error" => false

        );
    }

    /**
     * validate expected post commands
     *
     * @return array
     */
    private function _getExpectedPostCommands(): array
    {
        return array();
    }

    /**
     * validate expected pre commands
     *
     * @return array
     */
    private function _getExpectedPreCommands(): array
    {
        return array();
    }
}
